#ifndef _UMI_UMI_L_HXX_
#define _UMI_UMI_L_HXX_ 1

/**
 * \file umi_l.hxx
 * UMI Local Interface Service Provider
 *
 * %LEGAL%
 */

#include <iostream>
#include "umic/umi_driver.hxx"
#include "umi/umi_user.hxx"



namespace INdepUmi100 {

  class UmiLocal;
  /**
   * An UmiEnv (driver environment) designed for
   * the UmiLocal interface.
   */
  class UmiSync:public UmiEnv {
   public:
    /**
     * wait until there are messages at the input queue 
     * (or timeout). 
     * Depending on the implementation, the
     * wait my be active (no interrupt operation) or by a condition variable.
     *
     * \note syncGet() is called while a lock is already held (i.e. after a call to lock()). 
     * After return from syncGet() the lock is expected to be held.
     * Finally, unlock() will be called. 
     */
    virtual void syncGet(UmiLocal *)=0;

    /**
     * wait until there is room at the output queue 
     * (or timeout). 
     * Depending on the implementation, the
     * wait my be active (no interrupt operation) or by a condition variable.
     *
     * \note syncPut() is called while a lock is already held (i.e. after a call to lock()). 
     * After return from syncPut() the lock is expected to be held.
     * Finally, unlock() will be called. 
     */
    virtual void syncPut(UmiLocal *)=0;

    /**
     * lock the channel, disable concurrent threads including
     * interrupts. 
     *
     * \arg i is set to the iCtx construction argument of the calling UmiLocal object, 
     * when called from there. Any other value might be choosen for interrupt and others context.
     */
    virtual void lock(int i);

    /**
     * unlock the channel, enable concurrent threads including
     * interrupts
     *
     * \arg i is set to the iCtx construction argument of the calling UmiLocal object, 
     * when called from there. Any other value might be choosen for interrupt and others context.
     */
    virtual void unlock(int i);


    virtual Time tiNext( const Time &ti )=0;

    UmiSync();
    virtual ~UmiSync();
  };

  /**
   * Local Umi Channel.
   *
   * This class implements both, the Umi interface to the user libraries,
   * and the UmiChannel interface for the driver. Because the driver is implemented in the same address
   * space as the user libraries, the UmiLocal may be only used in environments without any kernel/user
   * separation.
   */
  class UmiLocal:public UmiChannel, public Umi {
    int _iCtx;
    UmiSync *_pus;


    UmiSync *pus() { return _pus; }

   public:

    /*
     * Umi interface
     */
    Failable &putMsg( const UmiControl*, const UmiData* );
    Failable &getMsg( UmiControlBuf*, UmiData *);
    Failable& xchMsg(UmiControl*pucd, const UmiData*pudd, UmiControlBuf *pucu, UmiData *pudu);

    /*
     * UmiChannel
     */
    Time tiNext( const Time &ti );		


    void init(UmiSync *pus) { _pus=pus; }
    void finiPool(Link *pli);

    /**
     * \arg iCtx is the context id (iCtx argument for various functions of UmiDriver, UmiChannel and UmiEnv) when these functions
     * are called from this object.
     *
     * \arg iChan is the channel number of this channel.
     */
    UmiLocal( int iChan, const UmiProto *puf, UmiSync *pus, int iCtx=0);
    ~UmiLocal();
  };
}

#endif
